@@ -56,8 +56,6 @@ cmd_build_tree(void)
       new = &cmd_root;
       while (*c)
 	{
-	  /* Walk through the keywords in the command from start of the command
-	   * to its end */
 	  char *d = c;
 	  while (*c && !isspace_(*c))
 	    c++;
@@ -67,12 +65,9 @@ cmd_build_tree(void)
 	      break;
 	  if (!new)
 	    {
-	      /* If new keyword is not stored yet */
 	      int size = sizeof(struct cmd_node) + c-d;
 	      new = malloc(size);
 	      bzero(new, size);
-	      /* Making this keyword child of the previons
-	       * keyword in the command */
 	      *old->plastson = new;
 	      old->plastson = &new->sibling;
 	      new->plastson = &new->son;
@@ -84,8 +79,6 @@ cmd_build_tree(void)
 	    c++;
 	}
 
-      /* In new there is now stored the last keyword in command.
-       * Only this command is going to contain cmd and help */
       if (cmd->is_real_cmd)
 	new->cmd = cmd;
       else
@@ -106,23 +99,12 @@ cmd_do_display_help(struct cmd_info *c)
 }
 
 static void
-cmd_display_help(struct cmd_node *m)
+cmd_display_help(struct cmd_info *c1, struct cmd_info *c2)
 {
-  /* Go through children of the base keyword. Each keyword has its own node. */
-  for (struct cmd_node *mm = m->son; mm; mm = mm->sibling)
-  {
-    /* Print hints for all of the children of given keyword. */
-    if (mm->help)
-      cmd_do_display_help(mm->help);
-    else if (mm->cmd)
-      cmd_do_display_help(mm->cmd);
-    else
-    {
-      /* This child does not contain hint, let's try its children instead
-       * (this child might be first part of multi keyword command) */
-      cmd_display_help(mm);
-    }
-  }
+  if (c1)
+    cmd_do_display_help(c1);
+  else if (c2)
+    cmd_do_display_help(c2);
 }
 
 static struct cmd_node *
@@ -159,7 +141,7 @@ cmd_list_ambiguous(struct cmd_node *root, char *cmd, int len)
 
   for(m=root->son; m; m=m->sibling)
     if (m->len > len && !memcmp(m->token, cmd, len))
-      cmd_display_help(m);
+      cmd_display_help(m->help, m->cmd);
 }
 
 void
@@ -191,15 +173,14 @@ cmd_help(char *cmd, int len)
 	break;
       n = m;
     }
-
-  if (n && n->cmd)
-    cmd_do_display_help(n->cmd);
+  cmd_display_help(n->cmd, NULL);
 
   /* Currently no help for options */
   if (n->final)
     return;
 
-  cmd_display_help(n);
+  for (m=n->son; m; m=m->sibling)
+    cmd_display_help(m->help, m->cmd);
 }
 
 static int
