@@ -259,8 +259,7 @@ global_commit(struct config *new, struct config *old)
       watchdog_warning,
       watchdog_timeout,
       gr_wait,
-      hostname,
-      shutdown
+      hostname
       );
 #undef COPY
 
