@@ -14,8 +14,9 @@
 #include "lib/ip.h"
 #include "lib/macro.h"
 #include "nest/route.h"
-#include "nest/attrs.h"
+#include "lib/attrs.h"
 #include "filter/data.h"
+#include "conf/conf.h"
 
 /* Possible return values of filter execution */
 enum filter_return {
@@ -51,18 +52,25 @@ struct filter {
 
 struct rte;
 
-enum filter_return f_run(const struct filter *filter, struct rte **rte, struct linpool *tmp_pool, int flags);
-enum filter_return f_run_args(const struct filter *filter, struct rte **rte, struct linpool *tmp_pool, uint argc, const struct f_val *argv, int flags);
-enum filter_return f_eval_rte(const struct f_line *expr, struct rte **rte, struct linpool *tmp_pool, uint argc, const struct f_val *argv, struct f_val *pres);
-enum filter_return f_eval_buf(const struct f_line *expr, struct linpool *tmp_pool, buffer *buf);
+enum filter_return f_run(const struct filter *filter, struct rte *rte, int flags);
+enum filter_return f_run_args(const struct filter *filter, struct rte *rte, uint argc, const struct f_val *argv, int flags);
+enum filter_return f_eval_rte(const struct f_line *expr, struct rte *rte, uint argc, const struct f_val *argv, uint resc, struct f_val *resv);
+enum filter_return f_eval_buf(const struct f_line *expr, buffer *buf);
 
-struct f_val cf_eval(const struct f_inst *inst, int type);
-static inline uint cf_eval_int(const struct f_inst *inst) { return cf_eval(inst, T_INT).val.i; };
+struct f_val cf_eval_tmp(const struct f_inst *inst, int type);
+static inline struct f_val *cf_eval(const struct f_inst *inst, int type)
+{
+  struct f_val val = cf_eval_tmp(inst, type);
+  return lp_val_copy(cfg_mem, &val);
+}
+
+static inline uint cf_eval_int(const struct f_inst *inst) { return cf_eval_tmp(inst, T_INT).val.i; };
 
 const char *filter_name(const struct filter *filter);
 int filter_same(const struct filter *new, const struct filter *old);
 int f_same(const struct f_line *f1, const struct f_line *f2);
 
+void filter_preconfig(struct config *new);
 void filter_commit(struct config *new, struct config *old);
 
 void filters_dump_all(struct dump_request *);
@@ -73,13 +81,4 @@ void filters_dump_all(struct dump_request *);
 
 #define FF_SILENT 2			/* Silent filter execution */
 
-/* Custom route attributes */
-struct custom_attribute {
-  resource r;
-  struct f_dynamic_attr *fda;
-  const char *name;
-};
-
-struct custom_attribute *ca_lookup(pool *p, const char *name, int ea_type);
-
 #endif
