@@ -111,8 +111,8 @@ t_bstrcmp(void)
   bt_assert(bstrcmp("aa", "bb") < 0);
   bt_assert(bstrcmp("bb", "aa") > 0);
   bt_assert(bstrcmp(NULL, NULL) == 0);
-  bt_assert(bstrcmp(NULL, "bb") == -1);
-  bt_assert(bstrcmp("bb", NULL) == 1);
+  bt_assert(bstrcmp(NULL, "bb") < 0);
+  bt_assert(bstrcmp("bb", NULL) > 0);
 
   return 1;
 }
