@@ -101,7 +101,7 @@ sl_allocz(slab *s)
 void
 sl_free(void *oo)
 {
-  struct sl_obj *o = SKIP_BACK(struct sl_obj, data, oo);
+  SKIP_BACK_DECLARE(struct sl_obj, o, data, oo);
 
   rem_node(&o->n);
   xfree(o);
@@ -118,7 +118,7 @@ slab_free(resource *r)
 }
 
 static void
-slab_dump(resource *r)
+slab_dump(resource *r, unsigned indent UNUSED)
 {
   slab *s = (slab *) r;
   int cnt = 0;
@@ -192,7 +192,7 @@ static struct resclass sl_class = {
   slab_memsize
 };
 
-#define SL_GET_HEAD(x)	((struct sl_head *) (((uintptr_t) (x)) & ~(page_size-1)))
+#define SL_GET_HEAD(x)	PAGE_HEAD(x)
 
 #define SL_HEAD_CHANGE_STATE(_s, _h, _from, _to) ({ \
     ASSERT_DIE(_h->state == slh_##_from); \
@@ -229,7 +229,7 @@ sl_new(pool *p, uint size)
       + sizeof(u32) * s->head_bitfield_len
       + align - 1)
     / align * align;
-  } while (s->objs_per_slab * size + s->head_size > page_size);
+  } while (s->objs_per_slab * size + s->head_size > (size_t) page_size);
 
   if (!s->objs_per_slab)
     bug("Slab: object too large");
@@ -238,6 +238,17 @@ sl_new(pool *p, uint size)
   return s;
 }
 
+/**
+ * sl_delete - destroy an existing Slab
+ * @s: slab
+ *
+ * This function destroys the given Slab.
+ */
+void sl_delete(slab *s)
+{
+  rfree(&s->r);
+}
+
 /**
  * sl_alloc - allocate an object from Slab
  * @s: slab
@@ -249,6 +260,7 @@ void *
 sl_alloc(slab *s)
 {
   struct sl_head *h;
+  ASSERT_DIE(DG_IS_LOCKED(resource_parent(&s->r)->domain));
 
 redo:
   if (!(h = s->partial_heads.first))
@@ -324,6 +336,7 @@ sl_free(void *oo)
 {
   struct sl_head *h = SL_GET_HEAD(oo);
   struct slab *s = h->slab;
+  ASSERT_DIE(DG_IS_LOCKED(resource_parent(&s->r)->domain));
 
 #ifdef POISON
   memset(oo, 0xdb, s->data_size);
@@ -447,4 +460,66 @@ slab_lookup(resource *r, unsigned long a)
   return NULL;
 }
 
+static const uint stonehenge_sizes[] = { 56, 112, 168, 288, 448, 800, 1344 };
+
+struct stonehenge {
+  pool *p;
+  slab *s[ARRAY_SIZE(stonehenge_sizes)];
+};
+
+sth_block
+sth_alloc(stonehenge *sth, uint size)
+{
+  for (uint i=0; i<ARRAY_SIZE(stonehenge_sizes); i++)
+    if (size <= stonehenge_sizes[i])
+    {
+      if (!sth->s[i])
+	sth->s[i] = sl_new(sth->p, stonehenge_sizes[i]);
+
+      return (sth_block) { .block = sl_alloc(sth->s[i]), };
+    }
+
+  return (sth_block) {
+    .block = mb_alloc(sth->p, size),
+    .large = 1,
+  };
+}
+
+sth_block
+sth_allocz(stonehenge *sth, uint size)
+{
+  sth_block b = sth_alloc(sth, size);
+  bzero(b.block, size);
+  return b;
+}
+
+void
+sth_free(sth_block b)
+{
+  if (b.large)
+    mb_free(b.block);
+  else
+    sl_free(b.block);
+}
+
+stonehenge *
+sth_new(pool *pp)
+{
+  stonehenge tmps = {
+    .p = rp_new(pp, pp->domain, "Stonehenge"),
+  };
+
+  stonehenge *s = sth_alloc(&tmps, sizeof(stonehenge)).block;
+  *s = tmps;
+  return s;
+}
+
+void sth_delete(stonehenge *s)
+{
+  pool *p = s->p;
+  sth_free((sth_block) { s });
+  rp_free(p);
+}
+
+
 #endif
