@@ -55,25 +55,8 @@ static inline void channel_reimport(struct channel *c, struct rt_feeding_request
   ev_send(proto_event_list(c->proto), &c->reimport_event);
 }
 
-static inline bool channel_reload(struct channel *c, struct rt_feeding_request *rfr)
-{
-  if ((c->in_keep & RIK_PREFILTER) == RIK_PREFILTER)
-  {
-    channel_reimport(c, rfr);
-    return true;
-  }
-  else
-    return c->proto->reload_routes(c, rfr);
-}
-
-static inline void channel_reload_roa(struct channel *c, struct rt_feeding_request *rfr)
-{
-  ASSERT_DIE(channel_reload(c, rfr));
-}
-
 static inline void channel_refeed(struct channel *c, struct rt_feeding_request *rfr)
 {
-  CALL(c->proto->refeed_begin, c, rfr);
   rt_export_refeed(&c->out_req, rfr);
 }
 
@@ -83,18 +66,10 @@ static inline int proto_is_done(struct proto *p)
 static inline int channel_is_active(struct channel *c)
 { return (c->channel_state != CS_DOWN); }
 
-static inline enum channel_reloadable channel_reloadable(struct channel *c)
+static inline int channel_reloadable(struct channel *c)
 {
-  /* Import table always reloads locally */
-  if ((c->in_keep & RIK_PREFILTER) == RIK_PREFILTER)
-    return CHANNEL_RELOADABLE_LOCALLY;
-
-  /* The protocol should indicate how the reload actually works */
-  if (c->proto->reload_routes)
-    return c->reloadable;
-
-  /* No reload possible */
-  return CHANNEL_RELOADABLE_NEVER;
+  return c->reloadable && c->proto->reload_routes
+      || ((c->in_keep & RIK_PREFILTER) == RIK_PREFILTER);
 }
 
 static inline void
@@ -256,7 +231,7 @@ proto_add_channel(struct proto *p, struct channel_config *cf)
 
   c->channel_state = CS_DOWN;
   c->last_state_change = current_time();
-  c->reloadable = CHANNEL_RELOADABLE_LOCALLY;
+  c->reloadable = 1;
 
   init_list(&c->roa_subscriptions);
 
@@ -478,7 +453,6 @@ struct roa_subscription {
   void (*refeed_hook)(struct channel *, struct rt_feeding_request *);
   struct lfjour_recipient digest_recipient;
   event update_event;
-  struct settle aspa_settle;
 };
 
 struct roa_reload_request {
@@ -537,33 +511,9 @@ channel_roa_changed(void *_s)
 	  s->c->proto->name, s->c->name);
 }
 
-static void
-channel_aspa_settle_hook(struct settle *set)
-{
-  SKIP_BACK_DECLARE(struct roa_subscription, s, aspa_settle, set);
-  s->refeed_hook(s->c, NULL);
-}
-
-static void
-channel_aspa_changed(void *_s)
-{
-  struct roa_subscription *s = _s;
-
-  bool seen = false;
-  for (
-      struct lfjour_item *it;
-      it = lfjour_get(&s->digest_recipient);
-      lfjour_release(&s->digest_recipient, it)
-      )
-    seen = true;
-
-  if (seen)
-    settle_kick(&s->aspa_settle, s->c->proto->loop);
-}
-
 static inline void (*channel_roa_reload_hook(int dir))(struct channel *, struct rt_feeding_request *)
 {
-  return dir ? channel_reload_roa : channel_refeed;
+  return dir ? channel_reimport : channel_refeed;
 }
 
 static int
@@ -585,67 +535,36 @@ channel_roa_subscribe(struct channel *c, rtable *tab, int dir)
   if (channel_roa_is_subscribed(c, tab, dir))
     return;
 
-  /* The ROA auxiliary table has the base type */
-  ASSERT_DIE((tab->addr_type == NET_IP4) || (tab->addr_type == NET_IP6));
+  rtable *aux = tab->config->roa_aux_table->table;
 
   struct roa_subscription *s = mb_allocz(c->proto->pool, sizeof(struct roa_subscription));
   *s = (struct roa_subscription) {
     .c = c,
-    .tab = tab,
+    .tab = aux,
     .refeed_hook = channel_roa_reload_hook(dir),
     .digest_recipient = {
       .target = proto_work_list(c->proto),
       .event = &s->update_event,
     },
     .update_event = {
-      .data = s,
       .hook = channel_roa_changed,
+      .data = s,
     },
   };
 
   add_tail(&c->roa_subscriptions, &s->roa_node);
 
-  RT_LOCK(tab, t);
+  RT_LOCK(aux, t);
   rt_lock_table(t);
   rt_setup_digestor(t);
   lfjour_register(&t->export_digest->digest, &s->digest_recipient);
 }
 
-static void
-channel_aspa_subscribe(struct channel *c, rtable *tab, int dir)
-{
-  if (channel_roa_is_subscribed(c, tab, dir))
-    return;
-
-  ASSERT_DIE(tab->addr_type == NET_ASPA);
-
-  struct settle_config sc = { .min = 300 MS, .max = 10 S, };
-  struct roa_subscription *s = mb_allocz(c->proto->pool, sizeof(struct roa_subscription));
-  *s = (struct roa_subscription) {
-    .c = c,
-    .tab = tab,
-    .refeed_hook = channel_roa_reload_hook(dir),
-    .digest_recipient = {
-      .target = proto_work_list(c->proto),
-      .event = &s->update_event,
-    },
-    .update_event = {
-      .data = s,
-      .hook = channel_aspa_changed,
-    },
-    .aspa_settle = SETTLE_INIT(&sc, channel_aspa_settle_hook, NULL),
-  };
-
-  add_tail(&c->roa_subscriptions, &s->roa_node);
-
-  RT_LOCK(tab, t);
-  rt_lock_table(t);
-  lfjour_register(&t->export_all.journal, &s->digest_recipient);
-}
-
 static void
 channel_roa_unsubscribe(struct roa_subscription *s)
 {
+  struct channel *c = s->c;
+
   RT_LOCKED(s->tab, t)
   {
     lfjour_unregister(&s->digest_recipient);
@@ -653,24 +572,25 @@ channel_roa_unsubscribe(struct roa_subscription *s)
   }
 
   ev_postpone(&s->update_event);
-  settle_cancel(&s->aspa_settle);
 
   rem_node(&s->roa_node);
   mb_free(s);
-}
 
+  channel_check_stopped(c);
+}
 
 static void
 channel_roa_subscribe_filter(struct channel *c, int dir)
 {
   const struct filter *f = dir ? c->in_filter : c->out_filter;
+  rtable *tab;
   int valid = 1, found = 0;
 
   if ((f == FILTER_ACCEPT) || (f == FILTER_REJECT))
     return;
 
   /* No automatic reload for non-reloadable channels */
-  if (dir && (channel_reloadable(c) != CHANNEL_RELOADABLE_LOCALLY))
+  if (dir && !channel_reloadable(c))
     valid = 0;
 
   struct filter_iterator fit;
@@ -681,17 +601,16 @@ channel_roa_subscribe_filter(struct channel *c, int dir)
     switch (fi->fi_code)
     {
     case FI_ROA_CHECK:
-      if (valid)
-	channel_roa_subscribe(c, fi->i_FI_ROA_CHECK.rtc->roa_aux_table->table, dir);
+      tab = fi->i_FI_ROA_CHECK.rtc->table;
+      if (valid) channel_roa_subscribe(c, tab, dir);
       found = 1;
       break;
 
     case FI_ASPA_CHECK_EXPLICIT:
-      if (valid)
-	channel_aspa_subscribe(c, fi->i_FI_ASPA_CHECK_EXPLICIT.rtc->table, dir);
+      tab = fi->i_FI_ASPA_CHECK_EXPLICIT.rtc->table;
+      if (valid) channel_roa_subscribe(c, tab, dir);
       found = 1;
       break;
-
     default:
       break;
     }
@@ -829,12 +748,12 @@ channel_check_stopped(struct channel *c)
       proto_send_event(c->proto, c->proto->event);
 
       break;
-
     case CS_PAUSE:
       if (c->obstacles || !EMPTY_LIST(c->roa_subscriptions))
 	return;
 
       ASSERT_DIE(rt_export_get_state(&c->out_req) == TES_DOWN);
+      ASSERT_DIE(!rt_export_feed_active(&c->reimporter));
 
       channel_set_state(c, CS_START);
       break;
@@ -1053,8 +972,6 @@ channel_set_state(struct channel *c, uint state)
 
     if (cs == CS_UP)
       channel_do_pause(c);
-
-    channel_check_stopped(c);
     break;
 
   case CS_STOP:
@@ -1064,7 +981,6 @@ channel_set_state(struct channel *c, uint state)
       channel_do_pause(c);
 
     channel_do_stop(c);
-    channel_check_stopped(c);
     break;
 
   case CS_DOWN:
@@ -1124,11 +1040,10 @@ channel_request_reload(struct channel *c, struct rt_feeding_request *cir)
   else
     CD(c, "Full import reload requested");
 
-  if (!channel_reload(c, cir))
-    if (cir && cir->prefilter.mode)
-      cli_msg(-15, "%s.%s: partial reload refused, please run full reload instead", c->proto->name, c->name);
-    else
-      bug("%s.%s: full reload refused");
+  if ((c->in_keep & RIK_PREFILTER) == RIK_PREFILTER)
+    channel_reimport(c, cir);
+  else if (! c->proto->reload_routes(c, cir))
+    cli_msg(-15, "%s.%s: partial reload refused, please run full reload instead", c->proto->name, c->name);
 }
 
 const struct channel_class channel_basic = {
@@ -1295,7 +1210,7 @@ channel_reconfigure(struct channel *c, struct channel_config *cf)
     channel_request_reload(c, NULL);
 
   if (export_changed)
-    channel_refeed(c, NULL);
+    channel_request_full_refeed(c);
 
 done:
   CD(c, "Reconfigured");
@@ -1880,7 +1795,7 @@ protos_do_commit(struct config *new, struct config *old, int type)
     protos_commit_request.phase--;
 
   /* If something is pending, the next round will be called asynchronously from proto_rethink_goal(). */
-  if (new->shutdown || !proto_rethink_goal_pending)
+  if (!proto_rethink_goal_pending)
     protos_do_commit(new, old, type);
 }
 
@@ -1938,7 +1853,7 @@ proto_rethink_goal(struct proto *p)
     proto_start(p);
 
 done:
-  if (goal_pending && !--proto_rethink_goal_pending && protos_commit_request.new)
+  if (goal_pending && !--proto_rethink_goal_pending)
     protos_do_commit(
 	protos_commit_request.new,
 	protos_commit_request.old,
@@ -2930,7 +2845,7 @@ proto_cmd_reload(struct proto *p, uintptr_t _prr, int cnt UNUSED)
 
       if (prr->dir & CMD_RELOAD_OUT)
 	if (c->out_req.name)
-	  channel_refeed(c, channel_create_reload_request(prr));
+	  rt_export_refeed(&c->out_req, channel_create_reload_request(prr));
     }
 
   cli_msg(-15, "%s: reloading", p->name);
