@@ -1037,13 +1037,18 @@ babel_send_update_(struct babel_iface *ifa, btime changed, struct fib *rtable)
 
     if (e->n.addr->type == NET_IP4)
     {
-      /* Always prefer IPv4 nexthop if set */
-      if (ipa_nonzero(ifa->next_hop_ip4))
+      /* Use IPv4 nexthop if available and preferred */
+      if (ipa_nonzero(ifa->next_hop_ip4) &&
+	  (ifa->cf->next_hop_prefer == BABEL_NHP_NATIVE))
         msg.update.next_hop = ifa->next_hop_ip4;
 
       /* Only send IPv6 nexthop if enabled */
       else if (ifa->cf->ext_next_hop)
         msg.update.next_hop = ifa->next_hop_ip6;
+
+      /* Fallback (prefer IPv6 but !ext_next_hop) */
+      else
+	msg.update.next_hop = ifa->next_hop_ip4;
     }
     else
       msg.update.next_hop = ifa->next_hop_ip6;
@@ -2206,11 +2211,13 @@ babel_router_id_format(const eattr *a, byte *buf, uint len)
 
 static struct ea_class ea_babel_metric = {
   .name = "babel_metric",
+  .legacy_name = "Babel.metric",
   .type = T_INT,
 };
 
 static struct ea_class ea_babel_router_id = {
   .name = "babel_router_id",
+  .legacy_name = "Babel.router_id",
   .type = T_OPAQUE,
   .readonly = 1,
   .format = babel_router_id_format,
@@ -2218,6 +2225,7 @@ static struct ea_class ea_babel_router_id = {
 
 static struct ea_class ea_babel_seqno = {
   .name = "babel_seqno",
+  .legacy_name = "Babel.seqno",
   .type = T_INT,
   .readonly = 1,
   .hidden = 1,
@@ -2423,9 +2431,6 @@ babel_kick_timer(struct babel_proto *p)
 static int
 babel_preexport(struct channel *C, struct rte *new)
 {
-  if (SHUTTING_DOWN)
-    return -1;
-
   if (new->src->owner != &C->proto->sources)
     return 0;
 
@@ -2458,10 +2463,6 @@ babel_rt_notify(struct proto *P, struct channel *c UNUSED, const net_addr *net,
   struct babel_proto *p = (void *) P;
   struct babel_entry *e;
 
-  /* Ignore everything on shutdown */
-  if (SHUTTING_DOWN)
-    return;
-
   if (new)
   {
     /* Update */
