@@ -1821,10 +1821,8 @@ bgp_withdraw_bucket(struct bgp_ptx_private *c, struct bgp_bucket *b)
     struct bgp_prefix *px = HEAD(b->prefixes);
 
     log(L_ERR "%s: - withdrawing %N", p->p.name, px->ni->addr);
-    ASSERT_DIE(px->cur == b);
     rem_node(&px->buck_node);
     add_tail(&wb->prefixes, &px->buck_node);
-    px->cur = wb;
   }
 }
 
@@ -2000,15 +1998,9 @@ bgp_done_prefix(struct bgp_ptx_private *c, struct bgp_prefix *px, struct bgp_buc
   bgp_free_prefix(c, px);
 }
 
-const char *
+void
 bgp_tx_resend(struct bgp_proto *p, struct bgp_channel *bc)
 {
-  if (p->enhanced_refresh)
-  {
-    bc->feed_state = BFS_REFRESHING;
-    bgp_schedule_packet(p->conn, bc, PKT_BEGIN_REFRESH);
-  }
-
   uint seen = 0;
   {
     BGP_PTX_LOCK(bc->tx, c);
@@ -2037,14 +2029,8 @@ bgp_tx_resend(struct bgp_proto *p, struct bgp_channel *bc)
 	  bc->c.proto->name, bc->c.name, seen);
 
   }
-
-  if (p->enhanced_refresh)
-    bc->feed_state = BFS_REFRESHED;
-
-  if (seen || p->enhanced_refresh)
+  if (seen)
     bgp_schedule_packet(p->conn, bc, PKT_UPDATE);
-
-  return p->enhanced_refresh ? "from export table by enhanced route refresh" : "from export table by simple refeed";
 }
 
 /*
@@ -2141,7 +2127,6 @@ bgp_init_pending_tx(struct bgp_channel *c)
   bpp->exporter = (struct rt_exporter) {
     .journal = {
       .loop = c->c.proto->loop,
-      .domain = dom.rtable,
       .item_size = sizeof(struct rt_export_item),
       .item_done = bgp_out_item_done,
     },
@@ -2230,10 +2215,6 @@ bgp_preexport(struct channel *C, rte *e)
   struct bgp_proto *src = bgp_rte_proto(e);
   struct bgp_channel *c = (struct bgp_channel *) C;
 
-  /* Reject everything on shutdown */
-  if (SHUTTING_DOWN)
-    return -1;
-
   /* Ignore non-BGP channels */
   if (C->class != &channel_bgp)
     return -1;
@@ -2445,10 +2426,6 @@ bgp_rt_notify(struct proto *P, struct channel *C, const net_addr *n, rte *new, c
   struct bgp_bucket *buck;
   struct rte_src *path;
 
-  /* Reject everything on shutdown */
-  if (SHUTTING_DOWN)
-    return;
-
   /* Ignore non-BGP channels */
   if (C->class != &channel_bgp)
     return;
