@@ -283,16 +283,13 @@ bgp_prepare_capabilities(struct bgp_conn *conn)
   if (p->cf->llgr_mode)
     caps->llgr_aware = 1;
 
-  rcu_read_lock();
-  struct global_runtime *gr = atomic_load_explicit(&global_runtime, memory_order_relaxed);
-  if (p->cf->enable_hostname && gr->hostname)
+  if (p->cf->enable_hostname && p->hostname)
   {
-    size_t length = strlen(gr->hostname);
+    size_t length = strlen(p->hostname);
     char *hostname = mb_allocz(p->p.pool, length+1);
-    memcpy(hostname, gr->hostname, length+1);
+    memcpy(hostname, p->hostname, length+1);
     caps->hostname = hostname;
   }
-  rcu_read_unlock();
 
   /* Allocate and fill per-AF fields */
   BGP_WALK_CHANNELS(p, c)
@@ -632,7 +629,7 @@ bgp_read_capabilities(struct bgp_conn *conn, byte *pos, int len)
       caps->enhanced_refresh = 1;
       break;
 
-    case 71: /* Long lived graceful restart capability, RFC draft */
+    case 71: /* Long lived graceful restart capability, RFC 9494 */
       if (cl % 7)
 	goto err;
 
@@ -1352,6 +1349,37 @@ bgp_update_next_hop_ip(struct bgp_export_state *s, eattr *a, ea_list **to)
     REJECT(NO_LABEL_STACK);
 }
 
+static uint
+bgp_prepare_link_local_next_hop(struct bgp_write_state *s, ip_addr *nh)
+{
+  /*
+   * We have link-local next-hop in nh[1]
+   *
+   * Possible variants:
+   * [ ::, fe80::XX ] - BIRD's default/internal (LLNH_NATIVE)
+   * [ fe80::XX ] - draft-white-linklocal-capability (LLNH_SINGLE)
+   * [ fe80::XX, fe80::XX ] - Used in JunoOS (LLNH_DOUBLE)
+   */
+
+  switch (s->llnh_format)
+  {
+  case LLNH_NATIVE:
+    return 32;
+
+  case LLNH_SINGLE:
+    nh[0] = nh[1];
+    return 16;
+
+  case LLNH_DOUBLE:
+    nh[0] = nh[1];
+    return 32;
+
+  default:
+    ASSERT(0);
+    return 32;
+  }
+}
+
 static uint
 bgp_encode_next_hop_ip(struct bgp_write_state *s, eattr *a, byte *buf, uint size UNUSED)
 {
@@ -1374,6 +1402,13 @@ bgp_encode_next_hop_ip(struct bgp_write_state *s, eattr *a, byte *buf, uint size
     return 4;
   }
 
+  /* Handle variants of link-local-only next hop */
+  if (ipa_zero(nh[0]) && (len == 32))
+  {
+    nh = alloca_copy(nh, 32);
+    len = bgp_prepare_link_local_next_hop(s, nh);
+  }
+
   put_ip6(buf, ipa_to_ip6(nh[0]));
 
   if (len == 32)
@@ -1450,6 +1485,13 @@ bgp_encode_next_hop_vpn(struct bgp_write_state *s, eattr *a, byte *buf, uint siz
     return 12;
   }
 
+  /* Handle variants of link-local-only next hop */
+  if (ipa_zero(nh[0]) && (len == 32))
+  {
+    nh = alloca_copy(nh, 32);
+    len = bgp_prepare_link_local_next_hop(s, nh);
+  }
+
   put_u64(buf, 0); /* VPN RD is 0 */
   put_ip6(buf+8, ipa_to_ip6(nh[0]));
 
@@ -1526,7 +1568,7 @@ bgp_decode_next_hop_none(struct bgp_parse_state *s UNUSED, byte *data UNUSED, ui
   /*
    * Although we expect no next hop and RFC 7606 7.11 states that attribute
    * MP_REACH_NLRI with unexpected next hop length is considered malformed,
-   * FlowSpec RFC 5575 4 states that next hop shall be ignored on receipt.
+   * FlowSpec RFC 8955 4 states that next hop shall be ignored on receipt.
    */
 
   return;
@@ -2580,6 +2622,7 @@ again:
     .mp_reach = (c->afi != BGP_AF_IPV4) || c->ext_next_hop,
     .as4_session = p->as4_session,
     .add_path = c->add_path_tx,
+    .llnh_format = c->cf->llnh_format,
     .mpls = c->desc->mpls,
   };
 
@@ -3007,8 +3050,10 @@ bgp_rx_route_refresh(struct bgp_conn *conn, byte *pkt, uint len)
   {
   case BGP_RR_REQUEST:
     BGP_TRACE(D_PACKETS, "Got ROUTE-REFRESH");
-    const char *info = bgp_begin_route_refresh(p, c);
-    BGP_TRACE(D_EVENTS, "Reloading %s %s", c->c.name, info);
+    if (c->tx_keep)
+      bgp_tx_resend(p, c);
+    else
+      rt_export_refeed(&c->c.out_req, NULL);
     break;
 
   case BGP_RR_BEGIN:
@@ -3170,9 +3215,6 @@ bgp_fire_tx(struct bgp_conn *conn)
 
       if (c->feed_state == BFS_LOADED)
       {
-	if (c->enhanced_rr_again)
-	  ev_send_loop(p->p.loop, &c->enhanced_rr_restart);
-
 	c->feed_state = BFS_NONE;
 	end = bgp_create_end_mark(c, pkt);
 	return bgp_send(conn, PKT_UPDATE, end - buf);
@@ -3180,9 +3222,6 @@ bgp_fire_tx(struct bgp_conn *conn)
 
       else if (c->feed_state == BFS_REFRESHED)
       {
-	if (c->enhanced_rr_again)
-	  ev_send_loop(p->p.loop, &c->enhanced_rr_restart);
-
 	c->feed_state = BFS_NONE;
 	end = bgp_create_end_refresh(c, pkt);
 	return bgp_send(conn, PKT_ROUTE_REFRESH, end - buf);
@@ -3227,8 +3266,6 @@ bgp_schedule_packet(struct bgp_conn *conn, struct bgp_channel *c, int type)
 
   if (c)
   {
-    ASSERT_DIE(c->index < p->channel_count);
-
     if (! conn->channels_to_send)
     {
       conn->last_channel = c->index;
