@@ -49,10 +49,6 @@ pipe_rt_notify(struct proto *P, struct channel *src_ch, const net_addr *n, rte *
   struct pipe_proto *p = (void *) P;
   struct channel *dst = (src_ch == p->pri) ? p->sec : p->pri;
 
-  /* Reject everything on shutdown */
-  if (SHUTTING_DOWN)
-    return;
-
   if (!new && !old)
     return;
 
@@ -74,10 +70,6 @@ pipe_preexport(struct channel *C, rte *e)
 {
   struct pipe_proto *p = (void *) C->proto;
 
-  /* Reject everything on shutdown */
-  if (SHUTTING_DOWN)
-    return -1;
-
   /* Avoid direct loopbacks */
   if (e->sender == C->in_req.hook)
     return -1;
