@@ -28,11 +28,14 @@
 #include "lib/resource.h"
 #include "lib/socket.h"
 #include "lib/event.h"
+#include "lib/locking.h"
 #include "lib/timer.h"
+#include "lib/tlists.h"
 #include "lib/string.h"
 #include "nest/route.h"
 #include "nest/protocol.h"
 #include "nest/iface.h"
+#include "nest/mpls.h"
 #include "nest/cli.h"
 #include "nest/locks.h"
 #include "conf/conf.h"
@@ -46,21 +49,38 @@
  *	Debugging
  */
 
+static void
+async_dump_report(struct dump_request *dr UNUSED, int state, const char *fmt, ...)
+{
+  va_list args;
+  va_start(args, fmt);
+  vlog(((state > 1000) ? L_ERR : L_INFO)[0], fmt, args);
+  va_end(args);
+}
+
+static void
+async_dump_run(struct dump_request *dreq)
+{
+  RDUMP("ASYNC STATE DUMP\n");
+
+  rdump(dreq, &root_pool);
+  sk_dump_all(dreq);
+  // XXXX tm_dump_all();
+  if_dump_all(dreq);
+  neigh_dump_all(dreq);
+  ea_dump_all(dreq);
+  rt_dump_all(dreq);
+  protos_dump_all(dreq);
+
+  debug("\n");
+}
+
 void
 async_dump(void)
 {
-  debug("INTERNAL STATE DUMP\n\n");
-
-  rdump(&root_pool);
-  sk_dump_all();
-  // XXXX tm_dump_all();
-  if_dump_all();
-  neigh_dump_all();
-  rta_dump_all();
-  rt_dump_all();
-  protos_dump_all();
-
-  debug("\n");
+  struct dump_request *dr = dump_to_file_init(0);
+  dr->report = async_dump_report;
+  dump_to_file_run(dr, "bird.dump", "async dump", async_dump_run);
 }
 
 /*
@@ -111,21 +131,21 @@ get_hostname(linpool *lp)
 #ifdef PATH_IPROUTE_DIR
 
 static inline void
-add_num_const(char *name, int val, const char *file, const uint line)
+add_num_const(struct config *conf, char *name, int val, const char *file, const uint line)
 {
   struct f_val *v = cfg_alloc(sizeof(struct f_val));
   *v = (struct f_val) { .type = T_INT, .val.i = val };
-  struct symbol *sym = cf_get_symbol(name);
-  if (sym->class && cf_symbol_is_local(sym))
+  struct symbol *sym = cf_get_symbol(conf, name);
+  if (sym->class && cf_symbol_is_local(conf, sym))
     cf_error("Error reading value for %s from %s:%d: already defined", name, file, line);
 
-  cf_define_symbol(sym, SYM_CONSTANT | T_INT, val, v);
+  cf_define_symbol(conf, sym, SYM_CONSTANT | T_INT, val, v);
 }
 
 /* the code of read_iproute_table() is based on
    rtnl_tab_initialize() from iproute2 package */
 static void
-read_iproute_table(char *file, char *prefix, uint max)
+read_iproute_table(struct config *conf, char *file, char *prefix, uint max)
 {
   char buf[512], namebuf[512];
   char *name;
@@ -162,7 +182,7 @@ read_iproute_table(char *file, char *prefix, uint max)
       if ((*p < 'a' || *p > 'z') && (*p < 'A' || *p > 'Z') && (*p < '0' || *p > '9') && (*p != '_'))
 	*p = '_';
 
-    add_num_const(namebuf, val, file, line);
+    add_num_const(conf, namebuf, val, file, line);
   }
 
   fclose(fp);
@@ -171,6 +191,8 @@ read_iproute_table(char *file, char *prefix, uint max)
 #endif // PATH_IPROUTE_DIR
 
 
+#define CONFIG_COMMIT(cf, ...)	({ CONFIG_REF_LOCAL(cr, cf); config_commit(&cr, __VA_ARGS__); })
+
 static char *config_name = PATH_CONFIG_FILE;
 
 static int
@@ -182,6 +204,8 @@ cf_read(byte *dest, uint len, int fd)
   return l;
 }
 
+static void cli_preconfig(struct config *c);
+
 void
 sysdep_preconfig(struct config *c)
 {
@@ -191,18 +215,25 @@ sysdep_preconfig(struct config *c)
   c->watchdog_warning = UNIX_DEFAULT_WATCHDOG_WARNING;
 
 #ifdef PATH_IPROUTE_DIR
-  read_iproute_table(PATH_IPROUTE_DIR "/rt_protos", "ipp_", 255);
-  read_iproute_table(PATH_IPROUTE_DIR "/rt_realms", "ipr_", 0xffffffff);
-  read_iproute_table(PATH_IPROUTE_DIR "/rt_scopes", "ips_", 255);
-  read_iproute_table(PATH_IPROUTE_DIR "/rt_tables", "ipt_", 0xffffffff);
+  read_iproute_table(c, PATH_IPROUTE_DIR "/rt_protos", "ipp_", 255);
+  read_iproute_table(c, PATH_IPROUTE_DIR "/rt_realms", "ipr_", 0xffffffff);
+  read_iproute_table(c, PATH_IPROUTE_DIR "/rt_scopes", "ips_", 255);
+  read_iproute_table(c, PATH_IPROUTE_DIR "/rt_tables", "ipt_", 0xffffffff);
 #endif
+
+  cli_preconfig(c);
 }
 
-int
-sysdep_commit(struct config *new, struct config *old UNUSED)
+static void cli_commit(struct config *new, struct config *old);
+
+void
+sysdep_commit(struct config *new, struct config *old)
 {
-  log_switch(0, &new->logfiles, new->syslog_name);
-  return 0;
+  if (!new->shutdown)
+    log_switch(0, &new->logfiles, new->syslog_name);
+
+  cli_commit(new, old);
+  bird_thread_commit(new, old);
 }
 
 static int
@@ -254,7 +285,7 @@ async_config(void)
       config_free(conf);
     }
   else
-    config_commit(conf, RECONFIG_HARD, 0);
+    CONFIG_COMMIT(conf, RECONFIG_HARD, 0);
 }
 
 static struct config *
@@ -335,7 +366,7 @@ cmd_reconfig(const char *name, int type, uint timeout)
   if (!conf)
     return;
 
-  int r = config_commit(conf, type, timeout);
+  int r = CONFIG_COMMIT(conf, type, timeout);
 
   if ((r >= 0) && (timeout > 0))
     {
@@ -394,14 +425,34 @@ cmd_reconfig_status(void)
  *	Command-Line Interface
  */
 
-static sock *cli_sk;
-static char *path_control_socket = PATH_CONTROL_SOCKET;
+static struct cli_config initial_control_socket_config = {
+  .name = PATH_CONTROL_SOCKET,
+  .mode = 0660,
+};
+#define path_control_socket initial_control_socket_config.name
 
+static struct cli_config *main_control_socket_config = NULL;
+
+#define TLIST_PREFIX cli_listener
+#define TLIST_TYPE struct cli_listener
+#define TLIST_ITEM n
+#define TLIST_WANT_ADD_TAIL
+#define TLIST_WANT_WALK
+static struct cli_listener {
+  TLIST_DEFAULT_NODE;
+  sock *s;
+  struct cli_config *config;
+} *main_control_socket = NULL;
+
+#include "lib/tlists.h"
+
+static TLIST_LIST(cli_listener) cli_listeners;
 
 static void
 cli_write(cli *c)
 {
-  sock *s = c->priv;
+  sock *s = c->sock;
+  ASSERT_DIE(c->sock);
 
   while (c->tx_pos)
     {
@@ -425,7 +476,9 @@ cli_write(cli *c)
 void
 cli_write_trigger(cli *c)
 {
-  sock *s = c->priv;
+  sock *s = c->sock;
+  if (!s)
+    return;
 
   if (s->tbuf == NULL)
     cli_write(c);
@@ -440,7 +493,8 @@ cli_tx(sock *s)
 int
 cli_get_command(cli *c)
 {
-  sock *s = c->priv;
+  sock *s = c->sock;
+  ASSERT_DIE(c->sock);
   byte *t = s->rbuf;
   byte *tend = s->rpos;
   byte *d = c->rx_pos;
@@ -479,16 +533,19 @@ cli_rx(sock *s, uint size UNUSED)
   return 0;
 }
 
+#define GLOBAL_CLI_DEBUG (atomic_load_explicit(&global_runtime, memory_order_relaxed)->cli_debug)
+
 static void
 cli_err(sock *s, int err)
 {
-  if (config->cli_debug)
+  if (GLOBAL_CLI_DEBUG)
     {
       if (err)
 	log(L_INFO "CLI connection dropped: %s", strerror(err));
       else
 	log(L_INFO "CLI connection closed");
     }
+
   cli_free(s->data);
 }
 
@@ -496,7 +553,7 @@ static void
 cli_connect_err(sock *s UNUSED, int err)
 {
   ASSERT_DIE(err);
-  if (config->cli_debug)
+  if (GLOBAL_CLI_DEBUG)
     log(L_INFO "Failed to accept CLI connection: %s", strerror(err));
 }
 
@@ -505,12 +562,12 @@ cli_connect(sock *s, uint size UNUSED)
 {
   cli *c;
 
-  if (config->cli_debug)
+  if (GLOBAL_CLI_DEBUG)
     log(L_INFO "CLI connect");
   s->rx_hook = cli_rx;
   s->tx_hook = cli_tx;
   s->err_hook = cli_err;
-  s->data = c = cli_new(s);
+  s->data = c = cli_new(s, ((struct cli_listener *) s->data)->config);
   s->pool = c->pool;		/* We need to have all the socket buffers allocated in the cli pool */
   s->fast_rx = 1;
   c->rx_pos = c->rx_buf;
@@ -518,33 +575,116 @@ cli_connect(sock *s, uint size UNUSED)
   return 1;
 }
 
-static void
-cli_init_unix(uid_t use_uid, gid_t use_gid)
+static struct cli_listener *
+cli_listen(struct cli_config *cf)
 {
-  sock *s;
-
-  cli_init();
-  s = cli_sk = sk_new(cli_pool);
+  struct cli_listener *l = mb_allocz(cli_pool, sizeof *l);
+  l->config = cf;
+  sock *s = l->s = sk_new(cli_pool);
   s->type = SK_UNIX_PASSIVE;
   s->rx_hook = cli_connect;
   s->err_hook = cli_connect_err;
+  s->data = l;
   s->rbsize = 1024;
   s->fast_rx = 1;
 
   /* Return value intentionally ignored */
-  unlink(path_control_socket);
+  unlink(cf->name);
 
-  if (sk_open_unix(s, path_control_socket) < 0)
-    die("Cannot create control socket %s: %m", path_control_socket);
+  if (sk_open_unix(s, &main_birdloop, cf->name) < 0)
+  {
+    log(L_ERR "Cannot create control socket %s: %m", cf->name);
+    return NULL;
+  }
 
-  if (use_uid || use_gid)
-    if (chown(path_control_socket, use_uid, use_gid) < 0)
-      die("chown: %m");
+  if (cf->uid || cf->gid)
+    if (chown(cf->name, cf->uid, cf->gid) < 0)
+    {
+      log(L_ERR "Cannot chown control socket %s: %m", cf->name);
+      return NULL;
+    }
 
-  if (chmod(path_control_socket, 0660) < 0)
-    die("chmod: %m");
+  if (chmod(cf->name, cf->mode) < 0)
+  {
+    log(L_ERR "Cannot chmod control socket %s: %m", cf->name);
+    return NULL;
+  }
+
+  cli_listener_add_tail(&cli_listeners, l);
+
+  return l;
 }
 
+static void
+cli_deafen(struct cli_listener *l)
+{
+  rfree(l->s);
+  unlink(l->config->name);
+  cli_listener_rem_node(&cli_listeners, l);
+  mb_free(l);
+}
+
+static void
+cli_init_unix(uid_t use_uid, gid_t use_gid)
+{
+  ASSERT_DIE(main_control_socket_config == NULL);
+
+  main_control_socket_config = &initial_control_socket_config;
+  main_control_socket_config->uid = use_uid;
+  main_control_socket_config->gid = use_gid;
+
+  ASSERT_DIE(main_control_socket == NULL);
+  main_control_socket = cli_listen(main_control_socket_config);
+  if (!main_control_socket)
+    die("Won't run without control socket");
+}
+
+static void
+cli_preconfig(struct config *c)
+{
+  if (!main_control_socket_config)
+    return;
+
+  struct cli_config *ccf = mb_alloc(cli_pool, sizeof *ccf);
+  memcpy(ccf, main_control_socket_config, sizeof *ccf);
+  ccf->n = (struct cli_config_node) {};
+  ccf->config = c;
+  cli_config_add_tail(&c->cli, ccf);
+}
+
+static void
+cli_commit(struct config *new, struct config *old)
+{
+  if (new->shutdown)
+  {
+    /* Keep the main CLI throughout the shutdown */
+    initial_control_socket_config.config = new;
+    main_control_socket->config = &initial_control_socket_config;
+  }
+
+  WALK_TLIST(cli_config, c, &new->cli)
+  {
+    _Bool seen = 0;
+    WALK_TLIST(cli_listener, l, &cli_listeners)
+      if (l->config->config != new)
+	if (!strcmp(l->config->name, c->name))
+	{
+	  ASSERT_DIE(l->config->config == old);
+	  l->config = c;
+	  seen = 1;
+	  break;
+	}
+
+    if (!seen)
+      cli_listen(c);
+  }
+
+  WALK_TLIST_DELSAFE(cli_listener, l, &cli_listeners)
+    if (l->config->config != new)
+      cli_deafen(l);
+}
+
+
 /*
  *	PID file
  */
@@ -622,7 +762,7 @@ void
 sysdep_shutdown_done(void)
 {
   unlink_pid_file();
-  unlink(path_control_socket);
+  cli_deafen(main_control_socket);
   log_msg(L_FATAL "Shutdown completed");
   exit(0);
 }
@@ -885,22 +1025,30 @@ main(int argc, char **argv)
     dmalloc_debug(0x2f03d00);
 #endif
 
-  parse_args(argc, argv);
-  log_switch(1, NULL, NULL);
-
-  random_init();
+  /* Prepare necessary infrastructure */
+  the_bird_lock();
+  times_update();
   resource_init();
-  timer_init();
+  random_init();
+
+  birdloop_init();
   olock_init();
-  io_init();
   rt_init();
+  io_init();
   if_init();
+  mpls_init();
 //  roa_init();
   config_init();
 
+  /* Arguments and logs */
+  parse_args(argc, argv);
+  log_switch(1, NULL, NULL);
+
   uid_t use_uid = get_uid(use_user);
   gid_t use_gid = get_gid(use_group);
 
+  cli_init();
+
   if (!parse_and_exit)
   {
     test_old_bird(path_control_socket);
@@ -923,6 +1071,8 @@ main(int argc, char **argv)
   if (parse_and_exit)
     exit(0);
 
+  flush_local_pages();
+
   if (!run_in_foreground)
     {
       pid_t pid = fork();
@@ -938,13 +1088,11 @@ main(int argc, char **argv)
       dup2(0, 2);
     }
 
-  main_thread_init();
-
   write_pid_file();
 
   signal_init();
 
-  config_commit(conf, RECONFIG_HARD, 0);
+  CONFIG_COMMIT(conf, RECONFIG_HARD, 0);
 
   graceful_restart_init();
 
